@@ -2,7 +2,7 @@ import http
 import typing
 import warnings
 
-__all__ = ("HTTPException",)
+__all__ = ("HTTPException", "WebSocketException")
 
 
 class HTTPException(Exception):
@@ -10,7 +10,7 @@ class HTTPException(Exception):
         self,
         status_code: int,
         detail: typing.Optional[str] = None,
-        headers: typing.Optional[dict] = None,
+        headers: typing.Optional[typing.Dict[str, str]] = None,
     ) -> None:
         if detail is None:
             detail = http.HTTPStatus(status_code).phrase
@@ -18,11 +18,27 @@ class HTTPException(Exception):
         self.detail = detail
         self.headers = headers
 
+    def __str__(self) -> str:
+        return f"{self.status_code}: {self.detail}"
+
     def __repr__(self) -> str:
         class_name = self.__class__.__name__
         return f"{class_name}(status_code={self.status_code!r}, detail={self.detail!r})"
 
 
+class WebSocketException(Exception):
+    def __init__(self, code: int, reason: typing.Optional[str] = None) -> None:
+        self.code = code
+        self.reason = reason or ""
+
+    def __str__(self) -> str:
+        return f"{self.code}: {self.reason}"
+
+    def __repr__(self) -> str:
+        class_name = self.__class__.__name__
+        return f"{class_name}(code={self.code!r}, reason={self.reason!r})"
+
+
 __deprecated__ = "ExceptionMiddleware"
 
 
