@@ -1,5 +1,4 @@
 import json
-import sys
 import typing
 from base64 import b64decode, b64encode
 
@@ -10,11 +9,6 @@ from starlette.datastructures import MutableHeaders, Secret
 from starlette.requests import HTTPConnection
 from starlette.types import ASGIApp, Message, Receive, Scope, Send
 
-if sys.version_info >= (3, 8):  # pragma: no cover
-    from typing import Literal
-else:  # pragma: no cover
-    from typing_extensions import Literal
-
 
 class SessionMiddleware:
     def __init__(
@@ -24,7 +18,7 @@ class SessionMiddleware:
         session_cookie: str = "session",
         max_age: typing.Optional[int] = 14 * 24 * 60 * 60,  # 14 days, in seconds
         path: str = "/",
-        same_site: Literal["lax", "strict", "none"] = "lax",
+        same_site: typing.Literal["lax", "strict", "none"] = "lax",
         https_only: bool = False,
     ) -> None:
         self.app = app
